'encoding UTF-8  Do not remove or change this line!
'*******************************************************************************
'*
'*  $RCSfile: formcontrols.inc,v $
'*
'*  $Revision: 1.4 $
'*
'*  last change: $Author: jsk $ $Date: 2005/06/03 07:17:42 $
'*
'*  The Contents of this file are made available subject to the terms of
'*  either of the following licenses
'*
'*         - GNU Lesser General Public License Version 2.1
'*         - Sun Industry Standards Source License Version 1.1
'*
'*  Sun Microsystems Inc., October, 2000
'*
'*  GNU Lesser General Public License Version 2.1
'*  =============================================
'*  Copyright 2000 by Sun Microsystems, Inc.
'*  901 San Antonio Road, Palo Alto, CA 94303, USA
'*
'*  This library is free software; you can redistribute it and/or
'*  modify it under the terms of the GNU Lesser General Public
'*  License version 2.1, as published by the Free Software Foundation.
'*
'*  This library is distributed in the hope that it will be useful,
'*  but WITHOUT ANY WARRANTY; without even the implied warranty of
'*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
'*  Lesser General Public License for more details.
'*
'*  You should have received a copy of the GNU Lesser General Public
'*  License along with this library; if not, write to the Free Software
'*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
'*  MA  02111-1307  USA
'*
'*
'*  Sun Industry Standards Source License Version 1.1
'*  =================================================
'*  The contents of this file are subject to the Sun Industry Standards
'*  Source License Version 1.1 (the License); You may not use this file
'*  except in compliance with the License. You may obtain a copy of the
'*  License at http://www.openoffice.org/license.html.
'*
'*  Software provided under this License is provided on an AS IS basis,
'*  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
'*  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
'*  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
'*  See the License for the specific provisions governing your rights and
'*  obligations concerning the Software.
'*
'*  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
'*
'*  Copyright: 2000 by Sun Microsystems, Inc.
'*
'*  All Rights Reserved.
'*
'*  Contributor(s): _______________________________________
'*
'*
'/******************************************************************************
' **
' ** owner : joerg.skottke@sun.com
' **
' ** short description : Tools to draw and select form controls in basic-ide
' **
'*******************************************************************************
' **
' #1 hGetControlParams    ' Delivers a number of dimensions for controls
' #1 hGetControlName      ' Delivers a (freeform) name to each control
' #1 hInsertControl       ' Insert a control to work with by index
' #1 hDrawControlOnDialog ' Set the coordinates and draw a control on a dialog
' #1 hDrawControl         ' finally draws the control using mousemovements
' #1 hGetControlPosXO     ' Retrieve the control position X-Orego
' #1 hGetControlPosYO     ' Retrieve the control position Y-Orego
' #1 hGetControlPosXE     ' Retrieve the control position X-End
' #1 hGetControlPosYE     ' Retrieve the control position Y-End
' #1 hGetControlPosXM     ' Retrieve the control position X-Median
' #1 hGetControlPosYM     ' Retrieve the control position Y-Median
' #1 hSelectControl       ' Select a control to work with by index
' **
'\******************************************************************************

public const ICONTROLCOUNT = 21

function hGetControlParams( cParam as string ) as integer

    '///<h1>Retrieve basic parameters to draw formcontrols to a dialog</h1>
    '///<ul>
    '///<li>Orego is always the upper left corner of a rectangle (control)</li>
    '///<li>Dist is the distance between the upper left corners of controls</li>
    '///<li>Size is width and heigth of a control</li>
    '///</ul>
    '///<b>All units are in percent of the windowsize!</b>
    
    ' current values are optimized for 1024x768 pixels screen resolution

    cParam = ucase( cParam ) 

    select case cParam
    case "XOREGO" : hGetControlParams() = 31
    case "XDIST"  : hGetControlParams() = 8
    case "XSIZE"  : hGetControlParams() = 6
    case "YOREGO" : hGetControlParams() = 30
    case "YDIST"  : hGetControlParams() = 7
    case "YSIZE"  : hGetControlParams() = 5    
    case else     : hGetControlParams() = 0
    end select
    
end function

'*******************************************************************************

function hGetControlName( iControl as integer ) as string

    '///<h1>A function to deliver a work-name for all controls</h1>
    '///The names are free form and can be changed to whatever you like

    dim sControl as string

    select case iControl
    case 1  : sControl = "PushButton"
    case 2  : sControl = "Image Control"
    case 3  : sControl = "CheckBox"
    case 4  : sControl = "RadioButton"
    case 5  : sControl = "Fixed Text"
    case 6  : sControl = "Edit Field"
    case 7  : sControl = "ListBox"
    case 8  : sControl = "ComboBox"
    case 9  : sControl = "Vertical ScrollBar"
    case 10 : sControl = "Horizontal ScrollBar"
    case 11 : sControl = "Frame"
    case 12 : sControl = "ProgressBar"
    case 13 : sControl = "Vertical Fixed Line"
    case 14 : sControl = "Horizontal Fixed Line"
    case 15 : sControl = "Date Field"
    case 16 : sControl = "Time Field"
    case 17 : sControl = "Numeric Field"
    case 18 : sControl = "Currency Field"
    case 19 : sControl = "Form Field"
    case 20 : sControl = "Pattern Field"
    case 21 : sControl = "File Control"
    end select
    
    hGetControlName() = sControl
    
end function

'*******************************************************************************

function hInsertControl( iControl as integer ) as string

    '///<h1>Function to insert one of the BASIC formcontrols by index</h1>
    '///<ul>
    '///<li>Verify the index passed to this function is valid</li>
    '///<li>Verify that the ToolsCollectionBar is available</li>
    '///<li>Click on the requested control button</li>
    '///<li>Return the name of the control</li>
    '///</ul>

    if ( ( iControl < 1 ) or ( iControl > ICONTROLCOUNT ) ) then
        warnlog( "Invalid control index passed" & iControl )
        hInsertControl() = "Internal error"
        exit function
    endif

    Kontext "ToolsCollectionBar"
    if ( not ToolsCollectionBar.exists() ) then
        warnlog( "The ToolsCollectionBar is not open" )
        hInsertControl() = "The ToolsCollectionBar is not open"
        exit function
    endif
    
    Kontext "ToolsCollectionBar"

    select case iControl
    case 1  : InsPushButton.click()
    case 2  : InsImgCtrl.click()
    case 3  : InsCheckbox.click()
    case 4  : InsRadioButton.click()
    case 5  : InsFixedText.click()
    case 6  : InsEditField.click()
    case 7  : InsListbox.click()
    case 8  : InsComboBox.click()
    case 9  : InsScrollBarV.click()
    case 10 : InsScrollBarH.click()
    case 11 : InsFrame.click()
    case 12 : InsProgressbar.click()
    case 13 : InsFixedLineV.click()
    case 14 : InsFixedLineH.click()
    case 15 : InsDateField.click()
    case 16 : InsTimeField.click()
    case 17 : InsNumField.click()
    case 18 : InsCurrencyField.click()
    case 19 : InsFormField.click()
    case 20 : InsPatternField.click()
    case 21 : InsFileCtrl.click()
    end select
    
    hInsertControl() = hGetControlName( iControl )
    
end function

'*******************************************************************************

function hDrawControlOnDialog( iControl as integer ) as string

    '///<h1>Helper function to draw a control on a dialog at a fixed position</h1>
    '///<ul>
    '///<li>Verify function parameters (this is a top-level function)</li>
    '///<li>Draw the control</li> 
    '///</ul>
    
    if ( ( iControl < 1 ) or ( iControl > ICONTROLCOUNT ) ) then
        warnlog( "Invalid control-number (iControl) passed: " & iControl )
        hDrawControlOnDialog() = false
        exit function
    endif    
    
    
    dim sControl as string   ' The name of the current control
    dim brc as boolean       ' some returnvalue
    
    ' coordinates of the controls on the dialog in the dialog-editor
    dim iXO as integer
    dim iYO as integer
    dim iXE as integer
    dim iYE as integer
    
    ' determine whare the control is to be painted
    iXO = hGetControlPosXO( iControl )
    iYO = hGetControlPosYO( iControl )
    iXE = hGetControlPosXE( iControl )
    iYE = hGetControlPosYE( iControl )
    
    ' click the desired control and return its name 
    sControl = hInsertControl( iControl )
    printlog( "Drawing: " & sControl & " at XO=" & iXO & _
                                          " XE=" & iXE & _
                                          " YO=" & iYO & _
                                          " YE=" & iYE )

    ' draw the control
    brc = hDrawControl( iXO, iYO, iXE, iYE )    
    hDrawControlOnDialog() = sControl
    
end function

'*******************************************************************************

function hDrawControl( xPos as integer, _
                       yPos as integer, _
                       xEnd as integer, _
                       yEnd as integer ) as boolean

   '///<h1>Function to draw a control on the dialog in the Dialog-Editor</h1>
   ' Note: Depending on the resolution the controls might be drawn outside of
   '       of the Dialog. This should be no problem.
   
   Kontext "BasicIDE"
   try
	   DialogWindow.MouseDown ( xPos, yPos )
	   DialogWindow.MouseMove ( xEnd, yEnd )
	   DialogWindow.MouseUp   ( xEnd, yEnd )
       hDrawControl() = true
   catch
       warnlog( "#i39852# Unable to complete mouseactions on dialog" )
       hDrawControl() = false
   endcatch

end function

'*******************************************************************************

function hGetControlPosXO( iControl as integer ) as integer

    '///<h1>Function to retrieve the upper left X-coordinate for a control</h1>

    dim xOffset as integer
        xOffset = hGetControlParams( "xorego" )
        
    dim xDistance as integer
        xDistance = hGetControlParams( "xdist" )
    
    ' Group the controls in four columns
    if ( iControl <= 6 ) then
    
        hGetControlPosXO() = xOffset
        
    elseif( ( iControl >= 7 ) and ( iControl <= 12 ) ) then
    
        hGetControlPosXO() = xOffset + 1 * xDistance
        
    elseif( ( iControl >= 13 ) and ( iControl <= 18 ) ) then
    
        hGetControlPosXO() = xOffset + 2 * xDistance
        
    else
    
        hGetControlPosXO() = xOffset + 3 * xDistance
        
    endif
    
end function
    
'*******************************************************************************

function hGetControlPosYO( iControl as integer ) as integer

    '///<h1>Function to retrieve the upper left Y-coordinate for a control</h1>

    dim yOffset as integer
        yOffset = hGetControlParams( "yorego" )
        
    dim yDistance as integer
        yDistance = hGetControlParams( "ydist" )
    
    select case iControl
    case 1 ,  7 , 13 , 19 : hGetControlPosYO() = yOffset
    case 2 ,  8 , 14 , 20 : hGetControlPosYO() = yOffset + 1 * yDistance
    case 3 ,  9 , 15 , 21 : hGetControlPosYO() = yOffset + 2 * yDistance
    case 4 , 10 , 16      : hGetControlPosYO() = yOffset + 3 * yDistance
    case 5 , 11 , 17      : hGetControlPosYO() = yOffset + 4 * yDistance
    case 6 , 12 , 18      : hGetControlPosYO() = yOffset + 5 * yDistance
    end select
    
end function

'*******************************************************************************

function hGetControlPosXE( iControl as integer ) as integer

    '///<h1>Function to retrieve the lower right X-coordinate for a control</h1>

    hGetControlPosXE() = hGetControlPosXO( iControl ) + _
                         hGetControlParams( "xsize" )
    
end function

'*******************************************************************************

function hGetControlPosYE( iControl as integer ) as integer

    '///<h1>Function to retrieve the lower right Y-coordinate for a control</h1>

    hGetControlPosYE() = hGetControlPosYO( iControl ) + _
                         hGetControlParams( "ysize" )
    
end function

'*******************************************************************************

function hGetControlPosXM( iControl as integer ) as integer

    '///<h1>Function to retrieve the middle (horizontal) of a control</h1>

    dim XO as integer
    dim XE as integer
    
    XO = hGetControlPosXO( iControl )
    XE = hGetControlPosXE( iControl )
    
    hGetControlPosXM() = XO + 0.5 * ( XE - XO )

end function 

'*******************************************************************************

function hGetControlPosYM( iControl as integer ) as integer

    '///<h1>Function to retrieve the middle (vertical) of a control</h1>

    dim YO as integer
    dim YE as integer
    
    YO = hGetControlPosYO( iControl )
    YE = hGetControlPosYE( iControl )
    
    hGetControlPosYM() = YO + 0.5 * ( YE - YO )  
    
end function

'*******************************************************************************

function hSelectControl( iControl as integer ) as boolean

    '///<h1>Function to select one of the BASIC formcontrols by index</h1>
    '///<ul>
    '///<li>Verify the index passed to this function is valid</li>
    '///<li>Verify that the ToolsCollectionBar is available</li>
    '///<li>Select the requested control by various methods</li>
    '///</ul>

    dim xPos as integer
    dim yPos as integer
    dim iTry as integer  ' we try three times to grab the control

    hSelectControl() = false

    if ( ( iControl < 1 ) or ( iControl > ICONTROLCOUNT ) ) then
        warnlog( "Incorrect index passed to function" )
        exit function
    endif       

    kontext "ToolsCollectionBar"
    if ( not ToolsCollectionBar.exists() ) then
        warnlog( "The ToolsCollectionBar is not visible, open it first" )
        exit function
    endif
    
    SelectMode.click()
    
    ' Note: The controls have areas where they ignore a mouseclick. E.g. the
    ' framecontrol can only be grabbed at the border, you won't be able to 
    ' select it by clicking in the middle. Furthermore - even if the Dialog-
    ' editor-window is maximized - we might still miss the upper left corner.
    ' So what happens in this loop is that we try to click in the middle of the
    ' control. This works in 19 out of 20 cases. If it fails, we try to grab the 
    ' border, first upper left corner then lower right. If this still fails,
    ' we try to select the control my using a rectangle selection around the
    ' control. If this last resort fails, the function exits gracefully but
    ' with a warnlog (causing some other functions to fail with warnings
    ' as well). Beware of possible problems with screen resolutions. 
    ' This function has been tested for 1024x768 and 1280x1024 pixels.
    ' Method 4 is dangerous because it might accidentially select the 
    ' background window which is the reason why this is not the default.
    
    for iTry = 1 to 4
        
        Kontext "BasicIDE"
        
        select case iTry
        case 1 
            xPos = hGetControlPosXM( iControl )  
            yPos = hGetControlPosYM( iControl )
            DialogWindow.MouseDown( xPos, yPos )
        case 2
            xPos = hGetControlPosXO( iControl )  
            yPos = hGetControlPosYO( iControl ) 
            DialogWindow.MouseDown( xPos, yPos )
        case 3
            xPos = hGetControlPosXE( iControl )  
            yPos = hGetControlPosYE( iControl )
            DialogWindow.MouseDown( xPos, yPos )
        case 4
            xPos = hGetControlPosXO( iControl ) - 2  
            yPos = hGetControlPosYO( iControl ) - 2
            DialogWindow.MouseDown( xPos, yPos )
            
            xPos = hGetControlPosXE( iControl ) + 2  
            yPos = hGetControlPosYE( iControl ) + 2              
            DialogWindow.MouseMove( xPos, yPos )
                          
        end select
        
        DialogWindow.MouseUp( xPos, yPos ) 
    
        kontext "ToolsCollectionBar"
        if ( ToolsCollectionBar.exists() ) then
            wait( 300 )
            if ( OpenProperties.isEnabled() ) then
                hSelectControl() = true
                exit for
            else
                wait( 500 )
            endif
        endif
        
    next iTry

end function


