'encoding UTF-8  Do not remove or change this line!
'*******************************************************************************
'*
'*  $RCSfile: key_tools.inc,v $
'*
'*  $Revision: 1.6 $
'*
'*  last change: $Author: tbo $ $Date: 2005/05/24 16:30:40 $
'*
'*  The Contents of this file are made available subject to the terms of
'*  either of the following licenses
'*
'*         - GNU Lesser General Public License Version 2.1
'*         - Sun Industry Standards Source License Version 1.1
'*
'*  Sun Microsystems Inc., October, 2000
'*
'*  GNU Lesser General Public License Version 2.1
'*  =============================================
'*  Copyright 2000 by Sun Microsystems, Inc.
'*  901 San Antonio Road, Palo Alto, CA 94303, USA
'*
'*  This library is free software; you can redistribute it and/or
'*  modify it under the terms of the GNU Lesser General Public
'*  License version 2.1, as published by the Free Software Foundation.
'*
'*  This library is distributed in the hope that it will be useful,
'*  but WITHOUT ANY WARRANTY; without even the implied warranty of
'*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
'*  Lesser General Public License for more details.
'*
'*  You should have received a copy of the GNU Lesser General Public
'*  License along with this library; if not, write to the Free Software
'*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
'*  MA  02111-1307  USA
'*
'*
'*  Sun Industry Standards Source License Version 1.1
'*  =================================================
'*  The contents of this file are subject to the Sun Industry Standards
'*  Source License Version 1.1 (the License); You may not use this file
'*  except in compliance with the License. You may obtain a copy of the
'*  License at http://www.openoffice.org/license.html.
'*
'*  Software provided under this License is provided on an AS IS basis,
'*  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
'*  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
'*  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
'*  See the License for the specific provisions governing your rights and
'*  obligations concerning the Software.
'*
'*  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
'*
'*  Copyright: 2000 by Sun Microsystems, Inc.
'*
'*  All Rights Reserved.
'*
'*  Contributor(s): _______________________________________
'*
'*
'/******************************************************************************
'*
'* owner : joerg.skottke@sun.com
'*
'* short description : tools for evaluation of key=value datalists
'*
'*******************************************************************************
' **
' #0 hGetDataPairAsString
' #0 hGetValueForPairAsString
' #0 hGetValueForPairAsLong
' #0 hGetValueForKeyAsInt
' #0 hGetKeyForPairAsString
' #0 hGetKeyForValueAsString
' **
'\******************************************************************************

' Some variables explained:
' 
' cComp is the KEY belonging to the VALUE above for reverse test
' 
' NOTE: Other related functions can be found in global/tools/t_filters.inc

'*******************************************************************************

function hGetDataPairAsString( sKey as string, sVal as long ) as string

   ' This function simply returns the concatenated string consisting of
   ' the key (usually the filename), an '=' sign and a value (typically the
   ' filesize). This is needed for listwrite which only takes a stringarray
   ' sVal (long) will be casted to char.

   hGetDataPairAsString() = sKey & "=" & sVal

end function

'*******************************************************************************

function hGetValueForPairAsString( cLine as string ) as string

   ' This function takes a string that (hopefully) contains one '='
   ' and returns the substringstring to the right from the '=' char.

   dim iCharPos as integer

   iCharPos = instr( cLine , "=" )
   iCharPos = len( cLine ) - iCharPos
   hGetValueForPairAsString() = right( cLine , iCharPos )

end function

'*******************************************************************************

function hGetValueForPairAsLong( cLine as string ) as long

   ' This function takes a string that (hopefully) contains one '='
   ' and returns the value to the right from it, which must be a number..

   dim iCharPos as integer

   iCharPos = instr( cLine , "=" )
   iCharPos = len( cLine ) - iCharPos
   hGetValueForPairAsLong = val( right( cLine , iCharPos ))

end function

'*******************************************************************************

function hGetValueForKeyAsInt( lsList() as string, sKey as string ) as integer

   ' This function returns the value of a key as string.
   ' The form of the input strings is 'key=value', the list is parsed
   ' The Value for the first occurrence of sKey is returned

   dim iItem as integer
   dim cString as string
   dim cComp as string

   hGetValueForKeyAsInt() = 0

   ' Scan through the list and look for sKey. If found, return the Value
   ' (everything to the right of the '=')
   for iItem = 1 to listcount( lsList() )
      if( instr( lsList( iItem ) , sKey ) <> 0 ) then
      
         cComp = hGetKeyforPairAsString( lsList( iItem ) )
         
         if( sKey = cComp ) then
            cString = hGetValueForPairAsString( lsList( iItem ) )
            iItem = listcount( lsList() ) + 1
         endif
         
      endif
   next iItem
   
   hGetValueForKeyAsInt() = val( cString )

end function

'*******************************************************************************

function hGetKeyForPairAsString( cLine as string ) as string

   ' This function returns the string to the left of the '='

   dim iCharPos as integer

   iCharPos = instr( cLine , "=" )

   ' get the string to the left of the = char
   if ( iCharPos > 0 ) then
      hGetKeyForPairAsString() = left( cLine , iCharPos -1 )
   else
      warnlog( "Invalid string passed to hGetKeyForPairAsString" )
      printlog( "It was: " & cLine )
   endif

end function

'*******************************************************************************

function hGetKeyForValueAsString( lsList() as string, sVal as string ) as string

   ' This function returns the value of a key as string.
   ' The form of the input strings is 'key=value', the list is parsed
   ' The key for the first occurrence of sVal is returned

   dim iCurrentValue as integer
   dim iListItems as integer
       iListItems = listcount( lsList() )
   dim cLine as string

   hGetKeyForValueAsString() = "Error: No matching KEY found for VALUE"

   ' scan through a list to find an entry that matches sKey. Return the Key.
   for iCurrentValue = 1 to iListItems
   
      cLine = lsList( iCurrentValue )
      
      if( instr( cLine , sVal ) <> 0 ) then
         hGetKeyForValueAsString() = hGetKeyForPairAsString( cLine )
         iCurrentValue = iListItems + 1
      end if
      
   next iCurrentValue

end function

'*******************************************************************************

function keycompare( found as string, expected as string, sKey as string, optional cBugID as string) as boolean
   if (isMissing(cBugID)) then 
       cBugID = ""
   endif
   if ( lcase(found) = lcase(expected) ) then
      printlog( "     * '" & sKey & "': ok" )
      keycompare = true
   else
      warnlog( "#" + cBugID + "# Control has incorrect value: '" & sKey & "'")
      printlog( " > Found...: '" & found  & "'" )
      printlog( " > Expected: '" & expected & "'" )
      keycompare = false
   endif
end function

'*******************************************************************************

function hGetValueForKeyAsLong( cLine as string ) as long
   ' This function takes a string that (hopefully) contains one '='
   ' and returns the value to the right from it, which must be a number..
   dim iCharPos as integer

   iCharPos = instr( cLine, "=" )
   iCharPos = len( cLine ) - iCharPos
   
   hGetValueForKeyAsLong  = val( right( cLine, iCharPos ))
end function

'*******************************************************************************

function hGetIndexForKeyAsInt( lsList() as string, sKey as string ) as integer

   dim iItem as integer
   dim cComp as string

   hGetIndexForNamedKeyAsInt() = 0

   ' Scan through the list and look for sKey. If found, return the Value
   ' (everything to the right of the '=')
   for iItem = 1 to listcount( lsList() )
      if( instr( lsList( iItem ) , sKey ) <> 0 ) then
      
         cComp = hGetKeyforValueAsString( lsList( iItem ) )
         
         if( sKey = cComp ) then
            hGetIndexForKeyAsInt() = iItem
            iItem = listcount( lsList() ) + 1
         endif
         
      endif
   next iItem

end function

'*******************************************************************************

