'encoding UTF-8  Do not remove or change this line!
'*******************************************************************************
'*
'*  $RCSfile: treelist_tools.inc,v $
'*
'*  $Revision: 1.1 $
'*
'*  last change: $Author: jsk $ $Date: 2005/06/23 13:48:58 $
'*
'*  The Contents of this file are made available subject to the terms of
'*  either of the following licenses
'*
'*         - GNU Lesser General Public License Version 2.1
'*         - Sun Industry Standards Source License Version 1.1
'*
'*  Sun Microsystems Inc., October, 2000
'*
'*  GNU Lesser General Public License Version 2.1
'*  =============================================
'*  Copyright 2000 by Sun Microsystems, Inc.
'*  901 San Antonio Road, Palo Alto, CA 94303, USA
'*
'*  This library is free software; you can redistribute it and/or
'*  modify it under the terms of the GNU Lesser General Public
'*  License version 2.1, as published by the Free Software Foundation.
'*
'*  This library is distributed in the hope that it will be useful,
'*  but WITHOUT ANY WARRANTY; without even the implied warranty of
'*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
'*  Lesser General Public License for more details.
'*
'*  You should have received a copy of the GNU Lesser General Public
'*  License along with this library; if not, write to the Free Software
'*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
'*  MA  02111-1307  USA
'*
'*
'*  Sun Industry Standards Source License Version 1.1
'*  =================================================
'*  The contents of this file are subject to the Sun Industry Standards
'*  Source License Version 1.1 (the License); You may not use this file
'*  except in compliance with the License. You may obtain a copy of the
'*  License at http://www.openoffice.org/license.html.
'*
'*  Software provided under this License is provided on an AS IS basis,
'*  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
'*  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
'*  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
'*  See the License for the specific provisions governing your rights and
'*  obligations concerning the Software.
'*
'*  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
'*
'*  Copyright: 2000 by Sun Microsystems, Inc.
'*
'*  All Rights Reserved.
'*
'*  Contributor(s): _______________________________________
'*
'*
'/******************************************************************************
'*
'* owner : joerg.skottke@sun.com
'*
'* short description :
'*
'*******************************************************************************
' **
' #1 hGetNodeCount        ' returns the number of visible nodes in a treelist
' #1 hSelectTopNode       ' jumps to the upper root-node in a treelist
' #1 hGetNodeName         ' gets the name of a specified node
' #1 hExpandNode          ' expands a specified node
' #1 hExpandAllNodes      ' expands all nodes in a treelist
' #1 hGetVisibleNodeNames ' feeds the names of all nodes into an array
' #1 hSelectNode          ' selects a node by index
' #1 hSelectTheLastNode   ' selects the absolute last node in a treelist
' **
'\******************************************************************************

function hGetNodeCount( _control as object ) as integer

   '///<h1>Tiny function to retrieve the number of visible nodes</h>

   hGetNodeCount() = _control.getItemCount()
   
end function

'*******************************************************************************

function hSelectTopNode( _control as object ) as boolean

    '///<h1>Tiny function to select the topmost rootnode in a treelist</h1>

   _control.typekeys( "<HOME>" )
   
   if ( _control.getSelIndex() = 1 ) then
   
      hSelectTopNode() = true
      
   else 
   
      hSelectTopNode() = false
      
   endif
   
end function

'*******************************************************************************

function hGetNodeName( _control as object , _id as integer ) as string

    '///<h1>Function to return the name of a specified node</li>

   _control.select( _id )
   hGetNodeName() = _control.getSelText()
   
end function

'*******************************************************************************

function hExpandNode( _control as object, node as integer ) as boolean

   '</h1>Function to expand a specified node</h1>

   dim iOldNodeCount as integer
   dim iNewNodeCOunt as integer
   
   iOldNodeCount = hGetNodeCount( _control )

   ' 0 = currently selected node, else open node 'node' (absolute)
   if ( node > 0 ) then
       _control.select( node )
   endif
   _control.typekeys( "<RIGHT>" )
   
   iNewNodeCount = hGetNodeCount( _control )
   
   if ( iNewNodeCount > iOldNodeCount ) then
   
      hExpandNode() = true
      
   else
   
      hExpandNode() = false
      
   endif
   
end function

'*******************************************************************************

function hExpandAllNodes( _control as object ) as integer

    '///<h1>Function to expand all nodes of a treelist</h1>

   dim iNode as integer
   dim iNodeCount as integer
   
   iNode = 0
   iNodeCount = 1000

   hSelectTopNode( _control )
   
   do while ( iNode < iNodeCount )
   
      iNode = iNode + 1
      hExpandNode( _control , iNode )
      iNodeCount = hGetNodeCount( _control )
      
   loop
       
   hExpandAllNodes() =  iNodeCount
      
end function

'*******************************************************************************

function hGetVisibleNodeNames( _control as object , _
                               lsList() as string ) as integer
                               
    '</h1>Function to store all nodenames into a list</h1>                               
            
   ' Get the list of all visible (expanded) nodes and store it
   ' in lsList(). if _id > ubound lsList() an error is given and lsList remains
   ' empty, thus returning "0"
                               
   dim iNodeCount as integer
   dim iThisNode as integer

   iNodeCount = hGetNodeCount( _control )
   
    if ( iNodeCount > ubound( lsList() ) ) then
        warnlog( "Array to small to hold: " & iNodeCount & " items" )
    else
        hSelectTopNode( _control )
   
        for iThisNode = 1 to iNodeCount

            listappend( lsList() , hGetNodeName( _control , iThisNode )

        next iThisNode
        
    endif
   
    hGetVisibleNodeNames() = listcount( lsList() )

end function  

'*******************************************************************************

function hSelectNode( _control as object , _id as integer ) as string

    '///<h1>Select a node by index</h1>

    if ( _id <= hGetNodeCount( _control ) ) then
        _control.select( _id )
        hSelectNode() = hGetNodeName( _control , _id )
    else
        warnlog( "Selected ID of node is too large" )
        hSelectNode() = ""
    endif

end function

'*******************************************************************************

function hSelectNodeByName( _control as object , _name as string ) as integer
                            
    '///<h1>Function to select a node by specifying its name</li>

    dim iThisNode as integer
    dim iNodeCount as integer    
    dim bBreak as boolean

    bBreak = false
    iThisNode = 0
    
    hExpandAllNodes( _control )
    
    ' First we try to jump to the node directly, if this fails we use the 
    ' slower but safer method (expand all nodes and step through)
    try
        _control.select( _name )
        hSelectNodeByName() = _control.getSelIndex()
        exit function
    catch
        printlog( "hSelectNodeByName::Using search method to find node" )
    endcatch
        
    hSelectTopNode( _control )

   
   ' find out how many objects are in the listbox
   iNodeCount = hGetNodeCount( _control )
                               
   ' Now find the first node that matches the given _name
   do while ( bBreak = false )
   
      iThisNode = iThisNode + 1
      
      if ( iThisNode > iNodeCount ) then 
         hSelectNodeByName() = 1
         bBreak = true
      else
         _control.select( iThisNode )
      endif
      
      if ( _name = _control.getText() ) then
         hSelectNodeByName() = iThisNode
         bBreak = true
      endif         
      
   loop
   
end function

'*******************************************************************************

function hSelectTheLastNode( _control as object ) as integer

    '///<h1>This function selects the last node in a treelist</h1>
    dim iPos as integer
    dim bNodeExpanded as boolean
        bNodeExpanded = true
    dim cNodeName as string
    
    while( bNodeExpanded )
                         
        iPos = hGetNodeCount( _control )
        cNodeName = hSelectNode( _control , iPos )
        printlog( "hSelectTheLastNode::" & cNodeName )
        bNodeExpanded = hExpandNode( _control , 0 )
    
    wend 

    hSelectTheLastNode() = iPos
    
end function
